@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2016 Fabien Siron <fabien.siron@epita.fr>
  * Copyright (c) 2016 Dmitry V. Levin <ldv@strace.io>
- * Copyright (c) 2016-2022 The strace developers.
+ * Copyright (c) 2016-2023 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -178,16 +178,20 @@ decode_nlmsg_type_netfilter(struct tcb *tcp, const struct xlat *const xlat,
 	const uint8_t subsys_id = (uint8_t) (type >> 8);
 	const uint8_t msg_type = (uint8_t) type;
 
+	tprint_flags_begin();
+	tprint_shift_begin();
 	printxval(xlat, subsys_id, dflt);
-
 	tprint_shift();
 	PRINT_VAL_U(8);
-	tprint_or();
+	tprint_shift_end();
+
+	tprint_flags_or();
 	if (subsys_id < ARRAY_SIZE(nf_nlmsg_types))
 		printxval(nf_nlmsg_types[subsys_id].xlat,
 			  msg_type, nf_nlmsg_types[subsys_id].dflt);
 	else
 		PRINT_VAL_X(msg_type);
+	tprint_flags_end();
 }
 
 typedef void (*nlmsg_types_decoder_t)(struct tcb *, const struct xlat *,
@@ -345,6 +349,7 @@ decode_nlmsg_flags_netfilter(const uint16_t type)
 		case NFT_MSG_GETOBJ:
 		case NFT_MSG_GETOBJ_RESET:
 		case NFT_MSG_GETFLOWTABLE:
+		case NFT_MSG_GETRULE_RESET:
 			return netlink_get_flags;
 		case NFT_MSG_DELTABLE:
 		case NFT_MSG_DELCHAIN:
@@ -447,8 +452,10 @@ decode_nlmsg_flags(const uint16_t flags, const uint16_t type,
 	} else if (family < ARRAY_SIZE(nlmsg_flags) && nlmsg_flags[family])
 		table = nlmsg_flags[family](type);
 
+	tprint_flags_begin();
 	printflags_ex(flags, "NLM_F_???", XLAT_STYLE_DEFAULT,
 		      netlink_flags, table, NULL);
+	tprint_flags_end();
 }
 
 static void
