@@ -157,14 +157,18 @@ print_perf_event_attr(struct tcb *const tcp, const kernel_ulong_t addr)
 		 */
 		tprint_struct_next();
 		tprints_field_name("config");
+		tprint_flags_begin();
 		if (attr->config >> 32) {
+			tprint_shift_begin();
 			PRINT_VAL_X(attr->config >> 32);
 			tprint_shift();
 			PRINT_VAL_U(32);
-			tprint_or();
+			tprint_shift_end();
+			tprint_flags_or();
 		}
 		printxval(perf_hw_id, attr->config & PERF_HW_EVENT_MASK,
 			   "PERF_COUNT_HW_???");
+		tprint_flags_end();
 		break;
 	case PERF_TYPE_SOFTWARE:
 		tprint_struct_next();
@@ -190,31 +194,43 @@ print_perf_event_attr(struct tcb *const tcp, const kernel_ulong_t addr)
 		 */
 		tprint_struct_next();
 		tprints_field_name("config");
+		tprint_flags_begin();
 		if (attr->config >> 32){
+			tprint_shift_begin();
 			PRINT_VAL_X(attr->config >> 32);
 			tprint_shift();
 			PRINT_VAL_U(32);
-			tprint_or();
+			tprint_shift_end();
+			tprint_flags_or();
 		}
 		if ((attr->config & PERF_HW_EVENT_MASK) >> 24) {
+			tprint_shift_begin();
 			PRINT_VAL_X((attr->config & PERF_HW_EVENT_MASK) >> 24);
 			tprint_shift();
 			PRINT_VAL_U(24);
-			tprint_or();
+			tprint_shift_end();
+			tprint_flags_or();
 		}
+		tprint_shift_begin();
 		printxval(perf_hw_cache_op_result_id,
 			  (attr->config >> 16) & 0xFF,
 			  "PERF_COUNT_HW_CACHE_RESULT_???");
 		tprint_shift();
 		PRINT_VAL_U(16);
-		tprint_or();
+		tprint_shift_end();
+
+		tprint_flags_or();
+		tprint_shift_begin();
 		printxval(perf_hw_cache_op_id, (attr->config >> 8) & 0xFF,
 			   "PERF_COUNT_HW_CACHE_OP_???");
 		tprint_shift();
 		PRINT_VAL_U(8);
-		tprint_or();
+		tprint_shift_end();
+
+		tprint_flags_or();
 		printxval(perf_hw_cache_id, attr->config & 0xFF,
 			  "PERF_COUNT_HW_CACHE_???");
+		tprint_flags_end();
 		break;
 	case PERF_TYPE_RAW:
 		/*
