@@ -6,7 +6,7 @@
  * Copyright (c) 1999 IBM Deutschland Entwicklung GmbH, IBM Corporation
  *                     Linux for s390 port by D.J. Barrow
  *                    <barrow_dj@mail.yahoo.com,djbarrow@de.ibm.com>
- * Copyright (c) 1999-2022 The strace developers.
+ * Copyright (c) 1999-2023 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -416,7 +416,7 @@ printnum_ ## name(struct tcb *const tcp, const kernel_ulong_t addr,	\
 	if (umove_or_printaddr(tcp, addr, &num))			\
 		return false;						\
 	tprint_indirect_begin();					\
-	tprintf(fmt, num);						\
+	tprintf_string(fmt, num);					\
 	tprint_indirect_end();						\
 	return true;							\
 }
@@ -443,9 +443,9 @@ printpair_ ## name(struct tcb *const tcp, const kernel_ulong_t addr,	\
 	if (umove_or_printaddr(tcp, addr, &pair))			\
 		return false;						\
 	tprint_array_begin();						\
-	tprintf(fmt, pair[0]);						\
+	tprintf_string(fmt, pair[0]);					\
 	tprint_array_next();						\
-	tprintf(fmt, pair[1]);						\
+	tprintf_string(fmt, pair[1]);					\
 	tprint_array_end();						\
 	return true;							\
 }
@@ -564,7 +564,7 @@ print_uuid(const unsigned char *uuid)
 		'\0'
 	};
 
-	tprints(str);
+	tprints_string(str);
 }
 
 enum sock_proto
@@ -630,7 +630,7 @@ static void
 print_string_in_angle_brackets(const char *str)
 {
 	tprint_associated_info_begin();
-	tprints(str);
+	tprints_string(str);
 	tprint_associated_info_end();
 }
 
@@ -647,33 +647,63 @@ printsocket(struct tcb *tcp, int fd, const char *path)
 	return true;
 }
 
-static bool
-printdev(struct tcb *tcp, int fd, const char *path)
+struct finfo *
+get_finfo_for_dev(const char *path, struct finfo *finfo)
 {
 	strace_stat_t st;
 
+	finfo->path = path;
+	finfo->type = FINFO_UNSET;
+	finfo->deleted = false;
+
 	if (path[0] != '/')
-		return false;
+		return finfo;
 
 	if (stat_file(path, &st)) {
 		debug_func_perror_msg("stat(\"%s\")", path);
-		return false;
+		return finfo;
 	}
 
 	switch (st.st_mode & S_IFMT) {
 	case S_IFBLK:
+		finfo->type = FINFO_DEV_BLK;
+		break;
 	case S_IFCHR:
+		finfo->type = FINFO_DEV_CHR;
+		break;
+	default:
+		return finfo;
+	}
+
+	finfo->dev.major = major(st.st_rdev);
+	finfo->dev.minor = minor(st.st_rdev);
+
+	return finfo;
+}
+
+static bool
+printdev(struct tcb *tcp, int fd, const char *path, const struct finfo *finfo)
+{
+	struct finfo finfo_buf;
+	if (!finfo)
+		finfo = get_finfo_for_dev(path, &finfo_buf);
+
+	switch (finfo->type) {
+	case FINFO_DEV_BLK:
+	case FINFO_DEV_CHR:
 		tprint_associated_info_begin();
-		print_quoted_string_ex(path, strlen(path),
+		print_quoted_string_ex(finfo->path, strlen(finfo->path),
 				       QUOTE_OMIT_LEADING_TRAILING_QUOTES,
 				       "<>");
 		tprint_associated_info_begin();
-		tprintf("%s %u:%u",
-			S_ISBLK(st.st_mode)? "block" : "char",
-			major(st.st_rdev), minor(st.st_rdev));
+		tprintf_string("%s %u:%u",
+			       (finfo->type == FINFO_DEV_BLK)? "block" : "char",
+			       finfo->dev.major, finfo->dev.minor);
 		tprint_associated_info_end();
 		tprint_associated_info_end();
 		return true;
+	default:
+		break;
 	}
 
 	return false;
@@ -724,7 +754,7 @@ printpidfd(pid_t pid_of_fd, int fd, const char *path)
 	pid_t pid = pidfd_get_pid(pid_of_fd, fd);
 	if (pid > 0) {
 		tprint_associated_info_begin();
-		tprints("pid:");
+		tprints_string("pid:");
 		/*
 		 * The pid translation is not needed because
 		 * the pid is in strace's namespace.
@@ -747,29 +777,31 @@ print_quoted_string_in_angle_brackets(const char *str, const bool deleted)
 	tprint_associated_info_end();
 
 	if (deleted)
-		tprints("(deleted)");
+		tprints_string("(deleted)");
 }
 
 void
-printfd_pid(struct tcb *tcp, pid_t pid, int fd)
+printfd_pid_with_finfo(struct tcb *tcp, pid_t pid, int fd, const struct finfo *finfo)
 {
 	PRINT_VAL_D(fd);
 
-	char path[PATH_MAX + 1];
+	char patha[PATH_MAX + 1];
 	bool deleted;
 	if (pid > 0 && !number_set_array_is_empty(decode_fd_set, 0)
-	    && getfdpath_pid(pid, fd, path, sizeof(path), &deleted) >= 0) {
+	    && (finfo || (getfdpath_pid(pid, fd, patha, sizeof(patha), &deleted) >= 0))) {
+		const char *path = finfo? finfo->path: patha;
 		if (is_number_in_set(DECODE_FD_SOCKET, decode_fd_set) &&
 		    printsocket(tcp, fd, path))
 			goto printed;
 		if (is_number_in_set(DECODE_FD_DEV, decode_fd_set) &&
-		    printdev(tcp, fd, path))
+		    printdev(tcp, fd, path, finfo))
 			goto printed;
 		if (is_number_in_set(DECODE_FD_PIDFD, decode_fd_set) &&
 		    printpidfd(pid, fd, path))
 			goto printed;
 		if (is_number_in_set(DECODE_FD_PATH, decode_fd_set))
-			print_quoted_string_in_angle_brackets(path, deleted);
+			print_quoted_string_in_angle_brackets(path,
+							      finfo? finfo->deleted: deleted);
 printed:	;
 	}
 
@@ -1065,7 +1097,7 @@ print_quoted_string_ex(const char *str, unsigned int size,
 	}
 
 	rc = string_quote(str, outstr, size, style, escape_chars);
-	tprints(outstr);
+	tprints_string(outstr);
 
 	if (((style & (QUOTE_0_TERMINATED | QUOTE_EXPECT_TRAILING_0))
 	     == (QUOTE_0_TERMINATED | QUOTE_EXPECT_TRAILING_0)) && rc) {
@@ -1105,7 +1137,7 @@ print_quoted_cstring(const char *str, unsigned int size)
  * Print path string specified by address `addr' and length `n'.
  * If path length exceeds `n', append `...' to the output.
  *
- * Returns the result of umovenstr.
+ * Returns the result of umovestr.
  */
 int
 printpathn(struct tcb *const tcp, const kernel_ulong_t addr, unsigned int n)
@@ -1152,7 +1184,7 @@ printpath(struct tcb *const tcp, const kernel_ulong_t addr)
  * Append `...' to the output if either the string length exceeds `max_strlen',
  * or QUOTE_0_TERMINATED bit is set and the string length exceeds `len'.
  *
- * Returns the result of umovenstr if style has QUOTE_0_TERMINATED,
+ * Returns the result of umovestr if style has QUOTE_0_TERMINATED,
  * or the result of umoven otherwise.
  */
 int
@@ -1218,7 +1250,7 @@ printstr_ex(struct tcb *const tcp, const kernel_ulong_t addr,
 		   && ((style & (QUOTE_0_TERMINATED | QUOTE_EXPECT_TRAILING_0))
 		       || len > max_strlen);
 
-	tprints(outstr);
+	tprints_string(outstr);
 	if (ellipsis)
 		tprint_more_data_follows();
 
@@ -1259,7 +1291,7 @@ print_nonzero_bytes(struct tcb *const tcp,
 		ret = false;
 	} else {
 		prefix_fun();
-		tprintf("/* bytes %u..%u */ ", start_offs, total_len - 1);
+		tprintf_string("/* bytes %u..%u */ ", start_offs, total_len - 1);
 
 		print_quoted_string(str, size, style);
 
@@ -1317,7 +1349,8 @@ dumpiov_upto(struct tcb *const tcp, const int len, const kernel_ulong_t addr,
 			data_size -= iov_len;
 			/* include the buffer number to make it easy to
 			 * match up the trace with the source */
-			tprintf(" * %" PRI_klu " bytes in buffer %d\n", iov_len, i);
+			tprintf_string(" * %" PRI_klu " bytes in buffer %d\n",
+				       iov_len, i);
 			dumpstr(tcp, iov_iov_base(i), iov_len);
 		}
 	}
@@ -1416,12 +1449,12 @@ dumpstr(struct tcb *const tcp, const kernel_ulong_t addr,
 				 * something already.
 				 */
 				if (i)
-					tprintf(" | <Cannot fetch %" PRI_klu
-						" byte%s from pid %d"
-						" @%#" PRI_klx ">\n",
-						fetch_size,
-						fetch_size == 1 ? "" : "s",
-						tcp->pid, addr + i);
+					tprintf_string(" | <Cannot fetch %" PRI_klu
+						       " byte%s from pid %d"
+						       " @%#" PRI_klx ">\n",
+						       fetch_size,
+						       fetch_size == 1 ? "" : "s",
+						       tcp->pid, addr + i);
 				return;
 			}
 			src = str;
@@ -1457,8 +1490,8 @@ dumpstr(struct tcb *const tcp, const kernel_ulong_t addr,
 			src++;
 		} while (++i & DUMPSTR_BYTES_MASK);
 
-		tprintf(" | %0*" PRI_klx "  %s |\n",
-			offs_chars, i - DUMPSTR_WIDTH_BYTES, outbuf);
+		tprintf_string(" | %0*" PRI_klx "  %s |\n",
+			       offs_chars, i - DUMPSTR_WIDTH_BYTES, outbuf);
 	}
 }
 
@@ -1694,10 +1727,16 @@ print_array_ex(struct tcb *const tcp,
 					     xlat_style);
 			}
 
-			tprint_array_index_end();
+			tprint_array_index_equal();
 		}
 
-		if (!print_func(tcp, elem_buf, elem_size, opaque_data)) {
+		bool break_needed =
+			!print_func(tcp, elem_buf, elem_size, opaque_data);
+
+		if (flags & PAF_PRINT_INDICES)
+			tprint_array_index_end();
+
+		if (break_needed) {
 			cur = end_addr;
 			break;
 		}
@@ -1760,10 +1799,12 @@ print_abnormal_hi(const kernel_ulong_t val)
 	if (current_klongsize > 4) {
 		const unsigned int hi = (unsigned int) ((uint64_t) val >> 32);
 		if (hi) {
+			tprint_shift_begin();
 			PRINT_VAL_X(hi);
 			tprint_shift();
 			PRINT_VAL_U(32);
-			tprint_or();
+			tprint_shift_end();
+			tprint_flags_or();
 		}
 	}
 }
