@@ -61,7 +61,7 @@ print_v4l2_rect(const MPERS_PTR_ARG(struct v4l2_rect *) const arg)
 #define PRINT_FIELD_FRACT(where_, field_)			\
 	do {							\
 		tprints_field_name(#field_);			\
-		STRACE_PRINTF("%u/%u",				\
+		tprintf_string("%u/%u",				\
 			      (where_).field_.numerator,	\
 			      (where_).field_.denominator);	\
 	} while (0)
@@ -503,15 +503,17 @@ print_v4l2_buffer_flags(uint32_t val)
 		return;
 	}
 
+	tprint_flags_begin();
 	if (flags) {
-		printflags(v4l2_buf_flags, flags, "V4L2_BUF_FLAG_???");
-		tprint_or();
+		printflags_in(v4l2_buf_flags, flags, "V4L2_BUF_FLAG_???");
+		tprint_flags_or();
 	}
 	printxval(v4l2_buf_flags_ts_type, ts_type,
 		  "V4L2_BUF_FLAG_TIMESTAMP_???");
-	tprint_or();
+	tprint_flags_or();
 	printxval(v4l2_buf_flags_ts_src, ts_src,
 		  "V4L2_BUF_FLAG_TSTAMP_SRC_???");
+	tprint_flags_end();
 }
 
 #define PRINT_FIELD_V4L2_BUFFER_FLAGS(where_, field_)		\
@@ -809,13 +811,14 @@ print_v4l2_cid(uint32_t cid, bool next_flags)
 		return;
 	}
 
+	tprint_flags_begin();
 	if (next_flags) {
 		uint32_t flags = cid & v4l2_control_query_flags->flags_mask;
 
 		if (flags) {
-			printflags(v4l2_control_query_flags, flags,
-				   "V4L2_CTRL_FLAG_NEXT_???");
-			tprint_or();
+			printflags_in(v4l2_control_query_flags, flags,
+				      "V4L2_CTRL_FLAG_NEXT_???");
+			tprint_flags_or();
 			cid &= ~flags;
 		}
 	}
@@ -824,6 +827,7 @@ print_v4l2_cid(uint32_t cid, bool next_flags)
 
 	if (id_name) {
 		print_xlat_ex(cid, id_name, XLAT_STYLE_DEFAULT);
+		tprint_flags_end();
 		return;
 	}
 
@@ -832,6 +836,7 @@ print_v4l2_cid(uint32_t cid, bool next_flags)
 
 	if (!class_str || (cid - class_id) >= 0x10000) {
 		print_xlat_ex(cid, "V4L2_CID_???", PXF_DEFAULT_STR);
+		tprint_flags_end();
 		return;
 	}
 
@@ -843,6 +848,7 @@ print_v4l2_cid(uint32_t cid, bool next_flags)
 
 	xsprintf(tmp_str, "%s+%#" PRIx64, class_str, cid - class_id);
 	print_xlat_ex(cid, tmp_str, XLAT_STYLE_DEFAULT);
+	tprint_flags_end();
 }
 
 #define PRINT_FIELD_V4L2_CID(where_, field_, next_)		\
