@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2017 JingPiao Chen <chenjingpiao@gmail.com>
- * Copyright (c) 2017-2021 The strace developers.
+ * Copyright (c) 2017-2023 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -39,7 +39,7 @@ main(void)
 		{ ENUM_KNOWN(0, IFLA_BRPORT_UNSPEC) },
 		{ ENUM_KNOWN(0x18, IFLA_BRPORT_FLUSH) },
 		{ ENUM_KNOWN(0x1a, IFLA_BRPORT_PAD) },
-		{ ARG_XLAT_UNKNOWN(0x27, "IFLA_BRPORT_???") },
+		{ ARG_XLAT_UNKNOWN(0x29, "IFLA_BRPORT_???") },
 		{ ARG_XLAT_UNKNOWN(0xbad, "IFLA_BRPORT_???") },
 	};
 	for (size_t i = 0; i < ARRAY_SIZE(un_attrs); i++) {
@@ -74,6 +74,8 @@ main(void)
 		{ ENUM_KNOWN(0x21, IFLA_BRPORT_ISOLATED) },
 		{ ENUM_KNOWN(0x23, IFLA_BRPORT_MRP_RING_OPEN) },
 		{ ENUM_KNOWN(0x24, IFLA_BRPORT_MRP_IN_OPEN) },
+		{ ENUM_KNOWN(0x27, IFLA_BRPORT_LOCKED) },
+		{ ENUM_KNOWN(0x28, IFLA_BRPORT_MAB) },
 	};
 	void *nlh_u8 = midtail_alloc(NLMSG_SPACE(hdrlen),
 				     NLA_HDRLEN * 2 + sizeof(uint8_t));
