@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2021 The strace developers.
+ * Copyright (c) 2016-2022 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -22,6 +22,7 @@
 #include <sys/vfs.h>
 #include <linux/fs.h>
 #include <linux/btrfs_tree.h>
+#include "print_utils.h"
 #include "xlat.h"
 
 #define XLAT_MACROS_ONLY
@@ -33,6 +34,7 @@
 #include "xlat/btrfs_balance_state.h"
 #include "xlat/btrfs_compress_types.h"
 #include "xlat/btrfs_cont_reading_from_srcdev_mode.h"
+#include "xlat/btrfs_csum_types.h"
 #include "xlat/btrfs_defrag_flags.h"
 #include "xlat/btrfs_dev_stats_values.h"
 #include "xlat/btrfs_dev_stats_flags.h"
@@ -46,6 +48,7 @@
 #include "xlat/btrfs_features_compat.h"
 #include "xlat/btrfs_features_compat_ro.h"
 #include "xlat/btrfs_features_incompat.h"
+#include "xlat/btrfs_fs_info_flags.h"
 #include "xlat/btrfs_key_types.h"
 
 #include <linux/fiemap.h>
@@ -115,7 +118,7 @@ sprint_makedev(unsigned long long val)
 
 #define ioc(x_) sprint_xlat_(x_, #x_)
 
-void
+static void
 prfl_btrfs(const struct xlat *xlat, const unsigned long long val,
 	   const char *str)
 {
@@ -126,7 +129,7 @@ prfl_btrfs(const struct xlat *xlat, const unsigned long long val,
 		printf(" */");
 }
 
-void
+static void
 prxval_btrfs(const struct xlat *xlat, const unsigned long long val,
 	     const char *str, bool known)
 {
@@ -150,15 +153,47 @@ print_uint64(const char *prefix, uint64_t val)
 	}
 }
 
+static void
+print_hex(uint8_t byte)
+{
+	printf("%c%c", BYTE_HEX_CHARS(byte));
+}
+
+static void
+print_uuid(const char * const prefix, const uint8_t * const uuid)
+{
+	size_t i = 0;
+
+	printf("%s", prefix);
+
+	for (; i < 4; i++)
+		print_hex(uuid[i]);
+	printf("-");
+
+	for (; i < 6; i++)
+		print_hex(uuid[i]);
+	printf("-");
+
+	for (; i < 8; i++)
+		print_hex(uuid[i]);
+	printf("-");
+
+	for (; i < 10; i++)
+		print_hex(uuid[i]);
+	printf("-");
+
+	for (; i < 16; i++)
+		print_hex(uuid[i]);
+}
+
 /* takes highest valid flag bit */
 static uint64_t
 max_flags_plus_one(int bit)
 {
-	int i;
 	uint64_t val = 0;
 	if (bit == -1)
 		return 1;
-	for (i = 0; i <= bit + 1 && i < 64; i++)
+	for (int i = 0; i <= bit + 1 && i < 64; ++i)
 		val |= (1ULL << i);
 	return val;
 }
@@ -228,9 +263,8 @@ btrfs_print_qgroup_inherit(struct btrfs_qgroup_inherit *inherit)
 	       inherit->lim.max_rfer, inherit->lim.max_excl,
 	       inherit->lim.rsv_rfer, inherit->lim.rsv_excl);
 	if (verbose) {
-		unsigned int i;
 		printf("qgroups=[");
-		for (i = 0; i < inherit->num_qgroups; i++) {
+		for (unsigned int i = 0; i < inherit->num_qgroups; ++i) {
 			if (i > 0)
 				printf(", ");
 			printf("%" PRI__u64, inherit->qgroups[i]);
@@ -370,7 +404,6 @@ btrfs_test_subvol_ioctls(void)
 	printf(") = -1 EBADF (%m)\n");
 
 	const unsigned int n_qgroups = 8;
-	unsigned int i;
 	struct btrfs_qgroup_inherit *inherit;
 	vol_args_v2.size =
 		sizeof(*inherit) + n_qgroups * sizeof(inherit->qgroups[0]);
@@ -380,7 +413,7 @@ btrfs_test_subvol_ioctls(void)
 	inherit->num_ref_copies = 0;
 	inherit->num_excl_copies = 0;
 	inherit->num_qgroups = n_qgroups;
-	for (i = 0; i < n_qgroups; i++)
+	for (unsigned int i = 0; i < n_qgroups; ++i)
 		inherit->qgroups[i] = 1ULL << i;
 	inherit->lim.flags = 0x7f;
 	inherit->lim.max_rfer = u64val;
@@ -416,16 +449,11 @@ btrfs_test_subvol_ioctls(void)
 	if (write_ok) {
 		struct btrfs_ioctl_vol_args_v2 args_passed;
 		long ret;
-		/*
-		 * Returns transid if flags & BTRFS_SUBVOL_CREATE_ASYNC
-		 * - BTRFS_IOC_SNAP_CREATE_V2
-		 * - BTRFS_IOC_SUBVOL_CREATE_V2
-		 */
 		int subvolfd;
 
 		strncpy(vol_args_v2.name, subvol_name,
 			sizeof(vol_args_v2.name));
-		vol_args_v2.flags = BTRFS_SUBVOL_CREATE_ASYNC;
+		vol_args_v2.flags = 0;
 		vol_args_v2.size = 0;
 		vol_args_v2.qgroup_inherit = NULL;
 		args_passed = vol_args_v2;
@@ -844,10 +872,9 @@ btrfs_print_tree_search_buf(struct btrfs_ioctl_search_key *key,
 			    void *buf, uint64_t buf_size)
 {
 	if (verbose) {
-		uint64_t i;
 		uint64_t off = 0;
 		printf("buf=[");
-		for (i = 0; i < key->nr_items; i++) {
+		for (uint64_t i = 0; i < key->nr_items; ++i) {
 			struct btrfs_ioctl_search_header *sh;
 			sh = (typeof(sh))(buf + off);
 			if (i)
@@ -1100,9 +1127,8 @@ btrfs_test_space_info_ioctl(void)
 		printf(" => {total_spaces=%" PRI__u64 ", ",
 			argsp->total_spaces);
 		if (verbose) {
-			unsigned int i;
 			printf("spaces=[");
-			for (i = 0; i < argsp->total_spaces; i++) {
+			for (unsigned int i = 0; i < argsp->total_spaces; ++i) {
 				struct btrfs_ioctl_space_info *info;
 				info = &argsp->spaces[i];
 				if (i)
@@ -1330,8 +1356,8 @@ btrfs_test_ino_path_ioctls(void)
 			fiemap->fm_mapped_extents);
 		if (verbose) {
 			printf("fm_extents=[");
-			unsigned int i;
-			for (i = 0; i < fiemap->fm_mapped_extents; i++) {
+			for (unsigned int i = 0;
+			     i < fiemap->fm_mapped_extents; ++i) {
 				struct fiemap_extent *fe;
 				fe = &fiemap->fm_extents[i];
 				if (i)
@@ -1339,7 +1365,7 @@ btrfs_test_ino_path_ioctls(void)
 				printf("{fe_logical=%" PRI__u64
 				       ", fe_physical=%" PRI__u64
 				       ", fe_length=%" PRI__u64
-				       ", ",
+				       ", fe_flags=",
 				       fe->fe_logical, fe->fe_physical,
 				       fe->fe_length);
 				prfl_btrfs(fiemap_extent_flags, fe->fe_flags,
@@ -1628,7 +1654,6 @@ btrfs_test_get_dev_stats_ioctl(void)
 	printf("}) = -1 EBADF (%m)\n");
 
 	if (write_ok) {
-		unsigned int i;
 		args.flags = BTRFS_DEV_STATS_RESET;
 		printf("ioctl(%d, %s, {devid=%s"
 			", nr_items=%" PRI__u64 ", flags=",
@@ -1642,18 +1667,18 @@ btrfs_test_get_dev_stats_ioctl(void)
 		prfl_btrfs(btrfs_dev_stats_flags, args.flags,
 			   "BTRFS_DEV_STATS_???");
 		printf(", [");
-		for (i = 0; i < args.nr_items; i++) {
+		for (unsigned int i = 0; i < args.nr_items; ++i) {
 			const char *name = xlookup(btrfs_dev_stats_values, i);
 			if (i)
 				printf(", ");
 
 			if (name) {
 				if (verbose_xlat)
-					printf("[%u /* %s */] = ", i, name);
+					printf("[%u /* %s */]=", i, name);
 				else
-					printf("[%s] = ", name);
+					printf("[%s]=", name);
 			} else {
-				printf("[%u] = ", i);
+				printf("[%u]=", i);
 			}
 
 			printf("%" PRI__u64, args.values[i]);
@@ -1955,8 +1980,7 @@ btrfs_test_read_ioctls(void)
 		XLAT(BTRFS_IOC_SUBVOL_GETFLAGS),
 	};
 
-	unsigned int i;
-	for (i = 0; i < ARRAY_SIZE(btrfs_read_cmd); ++i) {
+	for (unsigned int i = 0; i < ARRAY_SIZE(btrfs_read_cmd); ++i) {
 		ioctl(-1, (unsigned long) btrfs_read_cmd[i].val, 0);
 		printf("ioctl(-1, %s, NULL) = -1 EBADF (%m)\n",
 		       sprint_xlat_(btrfs_read_cmd[i].val,
@@ -1964,6 +1988,81 @@ btrfs_test_read_ioctls(void)
 	}
 }
 
+static void
+btrfs_print_fs_info_args(const struct btrfs_ioctl_fs_info_args * const args)
+{
+	print_uint64("{max_id=", args->max_id);
+	print_uint64(", num_devices=", args->num_devices);
+	print_uuid(", fsid=", args->fsid);
+	printf(", nodesize=%" PRIu32, args->nodesize);
+	printf(", sectorsize=%" PRIu32, args->sectorsize);
+	printf(", clone_alignment=%" PRIu32, args->clone_alignment);
+
+	if (args->flags & BTRFS_FS_INFO_FLAG_CSUM_INFO) {
+		printf(", csum_type=");
+		prxval_btrfs(btrfs_csum_types, args->csum_type,
+			     "BTRFS_CSUM_TYPE_???", true);
+		printf(", csum_size=%" PRIu16, args->csum_size);
+	}
+
+	printf(", flags=");
+	prfl_btrfs(btrfs_fs_info_flags, args->flags, "BTRFS_FS_INFO_FLAG_???");
+
+	if (args->flags & BTRFS_FS_INFO_FLAG_GENERATION)
+		print_uint64(", generation=", args->generation);
+
+	if (args->flags & BTRFS_FS_INFO_FLAG_METADATA_UUID)
+		print_uuid(", metadata_uuid=", args->metadata_uuid);
+
+	printf("}");
+}
+
+static void
+btrfs_fs_info_args_subtest(const struct btrfs_ioctl_fs_info_args * const args)
+{
+	ioctl(btrfs_test_dir_fd, BTRFS_IOC_FS_INFO, args);
+	printf("ioctl(%d, %s, ", btrfs_test_dir_fd, ioc(BTRFS_IOC_FS_INFO));
+	btrfs_print_fs_info_args(args);
+	printf(") = 0\n");
+}
+
+static void
+btrfs_test_fs_info_ioctl(void)
+{
+	struct btrfs_ioctl_fs_info_args args = {};
+
+	if (!write_ok)
+		return;
+
+	btrfs_fs_info_args_subtest(&args);
+
+	args.flags = BTRFS_FS_INFO_FLAG_CSUM_INFO;
+	btrfs_fs_info_args_subtest(&args);
+
+	args.flags = BTRFS_FS_INFO_FLAG_GENERATION;
+	btrfs_fs_info_args_subtest(&args);
+
+	args.flags = BTRFS_FS_INFO_FLAG_METADATA_UUID;
+	btrfs_fs_info_args_subtest(&args);
+
+	args.flags = BTRFS_FS_INFO_FLAG_CSUM_INFO
+		| BTRFS_FS_INFO_FLAG_GENERATION;
+	btrfs_fs_info_args_subtest(&args);
+
+	args.flags = BTRFS_FS_INFO_FLAG_CSUM_INFO
+		| BTRFS_FS_INFO_FLAG_METADATA_UUID;
+	btrfs_fs_info_args_subtest(&args);
+
+	args.flags = BTRFS_FS_INFO_FLAG_GENERATION
+		| BTRFS_FS_INFO_FLAG_METADATA_UUID;
+	btrfs_fs_info_args_subtest(&args);
+
+	args.flags = BTRFS_FS_INFO_FLAG_CSUM_INFO
+		| BTRFS_FS_INFO_FLAG_GENERATION
+		| BTRFS_FS_INFO_FLAG_METADATA_UUID;
+	btrfs_fs_info_args_subtest(&args);
+}
+
 static void
 rm_test_dir(void)
 {
@@ -2070,6 +2169,7 @@ main(int argc, char *argv[])
 	btrfs_test_dev_replace_ioctl();
 	btrfs_test_extent_same_ioctl();
 	btrfs_test_features_ioctls();
+	btrfs_test_fs_info_ioctl();
 
 	puts("+++ exited with 0 +++");
 
